clear
home

% %
% test reconstructing missing features
% 
tube = load('p1_FS_SS_CD56_CD16_CD3_CD8_CD4.txt');

% dataset
% FS SS CD56 | CD16 CD3 | CD8 CD4
did = {'FS', 'SS', 'CD56', 'CD16', 'CD3', 'CD8', 'CD4'};

N1 = 10000;
N2 = 10000;
tube_idx = randperm(size(tube,1));
x1t = tube(tube_idx(1:N1), :);
x2t = tube(tube_idx(N1+1:N1+N2), :);
xs  = tube(tube_idx(N1+N2+1:end), :);
d = 7;

x1 = nan(N1,d);
x1(:,[1 2 3 4 5]) = x1t(1:N1, [1 2 3 4 5]);
x2 = nan(N2,d);
x2(:,[1 2 3 6 7]) = x2t(1:N2, [1 2 3 6 7]);

% features
dp = false(d,1);    dp([1 2 3]) = true;     % common
dq = false(d,1);    dq([4 5]) = true;       % specific to x1
dr = false(d,1);    dr([6 7]) = true;       % specific to x2


% % 
% domain knowledge on mean
mm = [ ...
    800 680 240 350 200 170 200; ...    % granulocyte
    850 400 240 350 200 170 200; ...    % monocyte
    400 400 240 130 550 170 650; ...    % lymphocyte - helper T
    400 400 240 130 550 750 200; ...    % lymphocyte - cytotoxic T
    400 400 240 130 200 170 200; ...    % lymphocyte - B
    400 400 500 350 200 170 200; ...    % lymphocyte - NK
    ];
prior.mu_mean = mm;
clear mm


% % 
% cluster with ppca_md
ppca_md_K = size(prior.mu_mean,1);
ppca_md_q = 2;

fprintf('Training k=%d, q=%d \n', ppca_md_K, ppca_md_q);
tic;
ppca_md_param = ppca_md(x1, x2, [dp, dq, dr], ppca_md_K, ppca_md_q, prior);
ppca_md_time = toc;


% % 
% cluster - nn
fprintf('Cluster-NN imputation \n');
[x1_ppca, x2_ppca] = ppca_md_nn(x1, x2, [dp dq dr], ppca_md_param);


% % 
% nn
fprintf('NN imputation \n');
[x1_nn, x2_nn] = nn_impute(x1, x2, [dp dq dr], @util_dist2);


% % 
% evaluation - KL
fprintf('evaluation - KL \n');
xs1_nn = nn_impute(xs, x2, [dp dq dr], @util_dist2);
xs1_ppca = ppca_md_nn(xs, x2, [dp dq dr], ppca_md_param);

knlparam1 = mean(pdist(x1t))/20;
xs1_nn_kl = eval_kl(x1_nn, xs1_nn, xs, knlparam1);
xs1_ppca_kl = eval_kl(x1_ppca, xs1_ppca, xs, knlparam1);
fprintf('KL of NN : %2.2f \n', xs1_nn_kl);
fprintf('KL of Cluster-NN : %2.2f \n', xs1_ppca_kl);


% % 
% histograms
sid = [1 2 3 0; 4 5 6 7]';
plot_hist(1, tube, sid, did);
set(gcf, 'Position', [50 40 1100 400])


% %
% plot - true, nn, ppca mix
dpair = [ ...
    4 6; 4 7; 5 6; 5 7 ];
plot_imputation3(61, x1t, x1_nn, x1_ppca, dpair, did, {'True (file1)', 'NN', 'NN-Cluster'});
set(gcf, 'Position', [50 40 1100 800])

