
function [post, ll, logpdf] = ppca_md_E(x, mu, W, sig, pp)

N = size(x,1);
K = size(mu,1);

log_lh = zeros(N, K);

for k=1:K
    log_lh(:,k) = util_lnpost(x, mu(k,:), W(:,:,k), sig(k), pp(k));
end

[post, ll, logpdf] = util_post(log_lh);

end


function log_lh = util_lnpost(x, mu, W, sigma, p)

log_p = log(p);
[d,q] = size(W);
x0 = bsxfun(@minus, x, mu);

% C = sigma*eye(d) + W*W';
% logDetC = logDetM + (d-q)*log(sigma)
% x'*Cinv*x = (x'*x - x'*W*Minv*W'*x)/sigma
M = sigma*eye(q) + W'*W;
[R,f] = chol(M);
xRinv = x0 * W / R;
quadform = (sum(x0.^2,2) - sum(xRinv.^2,2))/sigma;
logSqrtDetC = sum(log(diag(R))) + (d-q)*log(sigma)/2;
log_lh = -0.5*quadform + (-logSqrtDetC + log_p) - d*log(2*pi)/2;

end % function util_lnpost


function [post, ll, logpdf] = util_post(log_lh)

maxll = max(log_lh,[],2);
post = exp(bsxfun(@minus, log_lh, maxll));
density = sum(post,2);
post = bsxfun(@rdivide, post, density);
logpdf = log(density) + maxll;
ll = sum(logpdf);

end % function util_post

